import type { Plugin } from 'vue'
import type { InstallConfig } from '#/index'
import type { AxiosError } from 'axios'
import { login, logout } from '@/security'
import { axiosRequestInterceptor } from '@/interceptor'

export const useCasAuthentication = () => ({
  axiosRequestInterceptor,
  login,
})

export const CasAuthentication: Plugin = {
  /**
   * Uses axios and the VusJS router to use CAS for login and logout :
   * - Login: redirects to the CAS login URL with the JWT server for the service.
   * - Logout: empties the local storage and redirect to the CAS logout URL.
   * - URL protected: checks if the URL has the meta "unistraCasAuthentication". If it's the meta use the login method.
   *
   * Documentation: https://git.unistra.fr/vue/cas-authentication
   *
   * @param {App} app - Vue instance.
   * @param {VueRouter} router - VueJS Router instance; Init the router with the logout URL, cf. documentation.
   * @param {Object} axios - Axios instance; Init this instance and add this to vue initialization.
   * @param {function} axios.post
   * @param {Object} options - Object containing the additional options.
   * @param {Boolean} options.appIsAllAuth - Option to indicate if the app is fully protected.
   * @param {String} options.authCasLogoutUrl - Name URL for logout. Init this route in the router.
   * @param {Boolean} options.hasFreeAPI - If the app has a free API, the axios request interceptor will not be added.
   * @param {String} options.jwtServerUrl - URL of JWT server
   * @param {String} options.serverCAS - URL of CAS server
   * @param {object} options.tokenErrorRoutes - error code as key, corresponding redirection url as value
   */
  install(app, { router, axios, options }: InstallConfig) {
    if (!router) throw new Error('You must install Vue-Router and pass an instance as a parameter of the CAS call.')
    if (!axios) throw new Error('You must install Axios and Vue-Axios and pass an instance as a parameter of the CAS call.')

    const jwtServerUrl = options?.jwtServerUrl ?? 'https://jwtserver-pprd.app.unistra.fr/api'
    const serverCAS = options?.serverCAS ?? 'https://cas.unistra.fr'
    const tokenErrorRoutes: Record<string, string> = {
      ...(options?.tokenErrorRoutes ?? {}),
      default: 'vue_cas_token_error',
    }
    const publicRouteNames = Object.values(tokenErrorRoutes)
    app.provide('casLogin', login)

    if (router.getRoutes().every(route => route.name !== 'vue_cas_token_error')) {
      router.addRoute({
        path: '/token/_error',
        name: 'vue_cas_token_error',
        component: app.component('VueCasTokenError', {
          template: `<div>no token retrieved, authentication failed</div>`,
        }),
      })
    }

    router.beforeEach(
      async (to, _from, next) => {
        if (to.name === options?.authCasLogoutUrl) {
          /**
           * If the name of the current route is equal to that passed in the Options object on the parameters,
           * the plugin call logout method.
           */
          logout(serverCAS)
          next(false)
          return
        }

        if (publicRouteNames.includes(to.name as string)) {
          /**
           * If route name is defined as an error route, plugin allow access.
           */
          next()
          return
        }

        if (!!to.query.service && !!to.query.ticket) {
          /**
           * If the URL has query string parameters after redirection from JWT Server, the plugin does an axios call
           * with the query string parameters in the data.
           * If the JWT Server sends the tokens, they are saved to local storage.
           * If the JWT Server sends an error, it tries to find route corresponding to error code then route to it,
           * or to default route named 'vue_cas_token_error'
           */
          try {
            const response = await axios.post(`${jwtServerUrl}/token/`, to.query)
            localStorage.setItem('JWT__access__token', response.data.access)
            localStorage.setItem('JWT__refresh__token', response.data.refresh)
            window.location.href = window.location.origin + window.location.pathname
            next(false)
            return
          } catch (error) {
            let name = tokenErrorRoutes.default
            if ((error as AxiosError).response?.status) {
              const status = (error as AxiosError).response?.status
              if (status && tokenErrorRoutes[`${status}`]) {
                name = tokenErrorRoutes[`${status}`]
              }
            }
            next({ name })
            return
          }
        }

        if (options?.appIsAllAuth || to.matched.some(record => record.meta.unistraCasAuthentication)) {
          /**
           * If the route has the meta "unistraCasAuthentication", the plugin checks if it's the tokens.
           * If not, the plugin call the login method.
           * If yes, the plugin lets the route pass.
           */
          if (!localStorage.getItem('JWT__refresh__token') || !localStorage.getItem('JWT__access__token')) {
            await login({ router, options, jwtServerUrl, serverCAS })
            next(false)
            return
          }
        }

        next()
      },
    )

    if (!options?.hasFreeAPI) {
      axios.interceptors.request.use(
        axiosRequestInterceptor(
          { axios, router, options, jwtServerUrl, serverCAS },
          false
        ),
        error => Promise.reject(error),
      )
    }
  },
}
