import type { Mock } from 'vitest'
import { afterEach, beforeEach, describe, expect, test, vi } from 'vitest'
import type { AxiosInterceptorConfig } from '#/index'
import type { AxiosInstance, InternalAxiosRequestConfig } from 'axios'
import { AxiosHeaders } from 'axios'
import { jwtServerUrl, jwtNotExpired, jwtExpired } from '~/general.fixtures'
import type { Router } from 'vue-router'
import { expired, login, refresh } from '@/security'
import { useCasAuthentication } from '../../../index'

vi.mock('@/security', () => ({
  refresh: vi.fn(),
  expired: vi.fn(),
  login: vi.fn(),
}))

describe('axiosRequestInterceptor', () => {
  let axiosRequestInterceptor: (config: AxiosInterceptorConfig,
                                inComposable?: boolean,
  ) => (config: InternalAxiosRequestConfig) => Promise<InternalAxiosRequestConfig>
  let axiosConfig: InternalAxiosRequestConfig

  beforeEach(() => {
    axiosRequestInterceptor = useCasAuthentication().axiosRequestInterceptor
    axiosConfig = {
      headers: new AxiosHeaders(),
      url: '/users/1',
      method: 'get',
      baseURL: 'https://form-local.dev:8080/api/v1',
    }
  })

  afterEach(() => {
    vi.resetAllMocks()
  })

  test('if the route contains "token", the config returned & the token not tested & bearer not added', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    })

    axiosConfig.url = '/token/refresh'

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBeUndefined()
    expect(expired).not.toBeCalled()
    expect(login).not.toBeCalled()
    expect(refresh).not.toBeCalled()
  })

  test('if the tokens not in the localStorage & not in composable, the config returned & the token not tested & bearer not added', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, false)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBeUndefined()
    expect(expired).not.toBeCalled()
    expect(login).not.toBeCalled()
    expect(refresh).not.toBeCalled()
  })

  test('if the tokens not in the localStorage & in composable, login used', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, true)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBeUndefined()
    expect(expired).not.toBeCalled()
    expect(login).toBeCalledTimes(1)
    expect(refresh).not.toBeCalled()
  })

  test('if the access token is in the localStorage & it is not expired, token added on the config', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, true)

    localStorage.setItem('JWT__access__token', jwtNotExpired)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBe(`Bearer ${jwtNotExpired}`)
    expect(expired).toBeCalledTimes(1)
    expect(login).not.toBeCalled()
    expect(refresh).not.toBeCalled()
  })

  test('if the access token is in the localStorage & it is expired & refresh token is not in the localStorage, login method used & token added on the config', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, true)

    localStorage.setItem('JWT__access__token', jwtExpired);
    (expired as Mock).mockReturnValue(true)
    localStorage.setItem('JWT__access__token', jwtNotExpired)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBe(`Bearer ${jwtNotExpired}`)
    expect(expired).toBeCalledTimes(1)
    expect(login).toBeCalledTimes(1)
    expect(refresh).not.toBeCalled()
  })

  test('if the access token is in the localStorage & it is expired & refresh token is in the localStorage & it is expired, login method used & token added on the config', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, true)

    localStorage.setItem('JWT__access__token', jwtExpired)
    localStorage.setItem('JWT__refresh__token', jwtExpired);
    (expired as Mock).mockReturnValue(true)
    localStorage.setItem('JWT__access__token', jwtNotExpired)
    localStorage.setItem('JWT__refresh__token', jwtNotExpired)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBe(`Bearer ${jwtNotExpired}`)
    expect(expired).toBeCalledTimes(2)
    expect(login).toBeCalledTimes(1)
    expect(refresh).not.toBeCalled()
  })

  test('if the access token is in the localStorage & it is expired & refresh token is in the localStorage & it is not expired, refresh method used & token added on the config', async () => {
    const interceptor = axiosRequestInterceptor({
      axios: {} as AxiosInstance,
      jwtServerUrl,
      router: {} as Router,
    }, true)

    localStorage.setItem('JWT__access__token', jwtExpired)
    localStorage.setItem('JWT__refresh__token', jwtNotExpired);
    (expired as Mock).mockReturnValueOnce(true)
                     .mockReturnValueOnce(false)
    localStorage.setItem('JWT__access__token', jwtNotExpired)

    const config = await interceptor(axiosConfig)

    expect(config.headers.Authorization).toBe(`Bearer ${jwtNotExpired}`)
    expect(expired).toBeCalledTimes(2)
    expect(login).not.toBeCalled()
    expect(refresh).toBeCalledTimes(1)
  })
})
