import { afterEach, beforeEach, describe, expect, test, vi } from 'vitest'
import { jwtServerUrl, resetLocation, serverCAS, spyLocation } from '~/general.fixtures'
import { login } from '@/security'
import { createMemoryHistory, createRouter } from 'vue-router'

const location = { ...window.location }

describe('login', () => {
  beforeEach(() => {
    spyLocation()
  })

  afterEach(() => {
    resetLocation(location)
  })

  test('if the loginUrl is undefined & the jwtServerUrl is undefined, an error is thrown', () => {
    expect(() => login({ serverCAS })).toThrowError(
      'You must provide the JWT Server URL and the CAS Server URL.',
    )
  })

  test('if the loginUrl is undefined & the serverCAS is undefined, an error is thrown', () => {
    expect(() => login({ jwtServerUrl })).toThrowError(
      'You must provide the JWT Server URL and the CAS Server URL.',
    )
  })

  test('if the loginUrl is undefined, the login method should redirect to the CAS login URL with the JWT server for the service', () => {
    login({ jwtServerUrl, serverCAS })

    let service = `${jwtServerUrl}/redirect/${window.btoa(window.location.href)}`
    expect(window.location.replace).toBeCalledWith(
      `${serverCAS}/cas/login?service=${encodeURIComponent(service)}`,
    )
  })

  test('if the loginUrl is defined & loginRouteIsInternal is false, the login method should redirect to the external URL', () => {
    login({
      options: {
        loginRoute: 'https://login.local',
        loginRouteIsInternal: false,
      },
    })

    expect(window.location.replace).toBeCalledWith('https://login.local')
  })

  test('if the loginUrl is defined & loginRouteIsInternal is undefined, the login method should redirect to the external URL', () => {
    login({
      options: {
        loginRoute: 'https://login.local',
      },
    })

    expect(window.location.replace).toBeCalledWith('https://login.local')
  })

  test('if the loginUrl is defined & loginRouteIsInternal is true & the router is undefined, an error is thrown', () => {
    expect(() => login({
      options: {
        loginRoute: 'https://login.local',
        loginRouteIsInternal: true,
      },
    })).toThrowError('If you want to use an internal route, you must provide the router instance.')
  })

  test('if the loginUrl is defined & loginRouteIsInternal is true & the router is defined, the router push method is called', () => {
    const router = createRouter({
      routes: [],
      history: createMemoryHistory('/'),
    })

    router.push = vi.fn(() => Promise.resolve())

    login({
      router,
      options: {
        loginRoute: '{name: "login"}',
        loginRouteIsInternal: true,
      },
    })

    expect(router.push).toBeCalledWith('{name: "login"}')
  })
})
