import type { LoginConfig } from '../types'
import type { AxiosError, AxiosInstance } from 'axios'

/**
 * If the loginRoute key is defined in the options:
 * - If loginRouteIsInternal is true, the user is redirected to the loginRoute page with vue router.
 * - If loginRouteIsInternal is false, the user is redirected to the loginRoute page with window.location.href.
 *
 * Else, redirects the user to login URL on the CAS service.
 * For the service: JWT Server URI + "/redirect/" + Current page encoded in Base64
 */
export const login = ({ router, options, serverCAS, jwtServerUrl }: LoginConfig): void => {
  localStorage.clear()
  if (!options?.loginRoute) {
    if (!jwtServerUrl || !serverCAS) throw new Error('You must provide the JWT Server URL and the CAS Server URL.')

    let service = `${jwtServerUrl}/redirect/${window.btoa(window.location.href)}`
    window.location.replace(
      `${serverCAS}/cas/login?service=${encodeURIComponent(service)}`,
    )
    return
  }

  if (!options?.loginRouteIsInternal) {
    window.location.replace(options?.loginRoute as string)
    return
  }

  if (!router) throw new Error('If you want to use an internal route, you must provide the router instance.')
  router.push(options?.loginRoute)
        .then(() => {
        })
}


/**
 * For logout, the plugin clears le local storage to delete the tokens.
 * Redirects the user to logout URL on the CAS service.
 */
export const logout = (serverCAS?: string): void => {
  if (!serverCAS) throw new Error('You must provide the CAS Server URL.')

  localStorage.clear()
  window.location.replace(`${serverCAS}/cas/logout`)
}

/**
 * Checks if the token is expired.
 * Recovers the payload, decodes the payload with Base64 decode method.
 * Checks the exp field with the current timestamp.
 */
export const expired = (token: string): boolean => {
  if (!token) return true
  return JSON.parse(window.atob(token.split('.')[1])).exp < Math.trunc(Date.now() / 1000)
}

/**
 * If the access token is expired, the plugin uses this method to refresh this token with the refresh token.
 * Do an axios call on the refresh URL to JWT Server with the refresh token in the data.
 *
 * @returns {Promise<void>}
 */
export const refresh = async (axios: AxiosInstance, jwtServerUrl?: string): Promise<void> => {
  if (!axios) throw new Error('You must provide the axios instance.')
  if (!jwtServerUrl) throw new Error('You must provide the JWT Server URL.')

  try {
    const response = await axios.post(
      `${jwtServerUrl}/token/refresh/`,
      { refresh: localStorage.getItem('JWT__refresh__token') },
    )
    localStorage.setItem('JWT__access__token', response.data.access)
  } catch (error) {
    throw new Error('Vue cas authentication plugin error : ' + (error as AxiosError).response?.data ?? 'refresh error')
  }
}
